@@ -14,7 +14,8 @@
 import argparse
 import textwrap
 
-from os_cloud_config import keystone_pki
+from os_cloud_config.cmd.utils import environment
+from os_cloud_config import ssl_pki
 
 
 def parse_args():
@@ -50,12 +51,15 @@ def parse_args():
                              'heat metadata file injected into image). '
                              'Different key/certs names and different '
                              'parent node are used (default: false)')
+    environment._add_logging_arguments(parser)
     return parser.parse_args()
 
 
 def main():
     args = parse_args()
+    environment._configure_logging(args)
+
     if args.heatenv:
-        keystone_pki.generate_certs_into_json(args.heatenv, args.seed)
+        ssl_pki.generate_cert_into_json(args.heatenv, "keystone")
     else:
-        keystone_pki.create_and_write_ca_and_signing_pairs(args.directory)
+        ssl_pki.create_and_write_ca_and_signing_pairs(args.directory)
