@@ -23,7 +23,7 @@ from os_cloud_config.cmd.utils import _clients as clients
 LOG = logging.getLogger(__name__)
 
 
-def register_nova_bm_node(service_host, node, client=None):
+def register_nova_bm_node(service_host, node, client=None, blocking=True):
     if not service_host:
         raise ValueError("Nova-baremetal requires a service host.")
     kwargs = {'pm_address': node["pm_addr"], 'pm_user': node["pm_user"]}
@@ -45,17 +45,23 @@ def register_nova_bm_node(service_host, node, client=None):
                                               node["mac"][0], **kwargs)
             break
         except (novaexc.ConnectionRefused, novaexc.ServiceUnavailable):
-            LOG.debug('Service not available, sleeping for 10 seconds.')
-            time.sleep(10)
+            if blocking:
+                LOG.debug('Service not available, sleeping for 10 seconds.')
+                time.sleep(10)
+            else:
+                LOG.debug('Service not available.')
     else:
-        LOG.debug('Service unavailable after 10 minutes, giving up.')
+        if blocking:
+            LOG.debug('Service unavailable after 10 minutes, giving up.')
+        else:
+            LOG.debug('Service unavailable after 60 tries, giving up.')
         raise novaexc.ServiceUnavailable()
     for mac in node["mac"][1:]:
         client.baremetal.add_interface(bm_node, mac)
     return bm_node
 
 
-def register_ironic_node(service_host, node, client=None):
+def register_ironic_node(service_host, node, client=None, blocking=True):
     properties = {"cpus": node["cpu"],
                   "memory_mb": node["memory"],
                   "local_gb": node["disk"],
@@ -65,18 +71,20 @@ def register_ironic_node(service_host, node, client=None):
                        "ipmi_username": node["pm_user"],
                        "ipmi_password": node["pm_password"]}
     elif node["pm_type"] == "pxe_ssh":
+        if "pm_virt_type" not in node:
+            node["pm_virt_type"] = "virsh"
         driver_info = {"ssh_address": node["pm_addr"],
                        "ssh_username": node["pm_user"],
                        "ssh_key_contents": node["pm_password"],
-                       "ssh_virt_type": "virsh"}
+                       "ssh_virt_type": node["pm_virt_type"]}
     elif node["pm_type"] == "pxe_iboot":
         driver_info = {"iboot_address": node["pm_addr"],
                        "iboot_username": node["pm_user"],
                        "iboot_password": node["pm_password"]}
         # iboot_relay_id and iboot_port are optional
-        if "iboot_relay_id" in node:
+        if "pm_relay_id" in node:
             driver_info["iboot_relay_id"] = node["pm_relay_id"]
-        if "iboot_port" in node:
+        if "pm_port" in node:
             driver_info["iboot_port"] = node["pm_port"]
     else:
         raise Exception("Unknown pm_type: %s" % node["pm_type"])
@@ -90,10 +98,16 @@ def register_ironic_node(service_host, node, client=None):
                                              properties=properties)
             break
         except (ironicexp.ConnectionRefused, ironicexp.ServiceUnavailable):
-            LOG.debug('Service not available, sleeping for 10 seconds.')
-            time.sleep(10)
+            if blocking:
+                LOG.debug('Service not available, sleeping for 10 seconds.')
+                time.sleep(10)
+            else:
+                LOG.debug('Service not available.')
     else:
-        LOG.debug('Service unavailable after 10 minutes, giving up.')
+        if blocking:
+            LOG.debug('Service unavailable after 10 minutes, giving up.')
+        else:
+            LOG.debug('Service unavailable after 60 tries, giving up.')
         raise ironicexp.ServiceUnavailable()
 
     for mac in node["mac"]:
@@ -140,20 +154,23 @@ def _get_node_id(node, node_map):
             return node_map['pm_addr'][node['pm_addr']]
 
 
-def _create_or_register_bm_node(service_host, node, node_map, client=None):
+def _update_or_register_bm_node(service_host, node, node_map, client=None,
+                                blocking=True):
     bm_id = _get_node_id(node, node_map)
     if bm_id:
         bm_node = client.baremetal.get(bm_id)
     else:
         bm_node = None
     if bm_node is None:
-        bm_node = register_nova_bm_node(service_host, node, client)
+        bm_node = register_nova_bm_node(service_host, node, client,
+                                        blocking=blocking)
     else:
         LOG.warning('Node %d already registered, skipping.' % bm_node.id)
     return bm_node.id
 
 
-def _create_or_register_ironic_node(service_host, node, node_map, client=None):
+def _update_or_register_ironic_node(service_host, node, node_map, client=None,
+                                    blocking=True):
     node_uuid = _get_node_id(node, node_map)
     massage_map = {'cpu': '/properties/cpus',
                    'memory': '/properties/memory_mb',
@@ -172,7 +189,8 @@ def _create_or_register_ironic_node(service_host, node, node_map, client=None):
     else:
         ironic_node = None
     if ironic_node is None:
-        ironic_node = register_ironic_node(service_host, node, client)
+        ironic_node = register_ironic_node(service_host, node, client,
+                                           blocking=blocking)
     else:
         LOG.debug('Node %s already registered, updating details.' % (
             ironic_node.uuid))
@@ -208,7 +226,8 @@ def _clean_up_extra_nodes(ironic_in_use, seen, client, remove=False):
             LOG.debug('Extra registered node %s found.' % node)
 
 
-def register_all_nodes(service_host, nodes_list, client=None, remove=False):
+def register_all_nodes(service_host, nodes_list, client=None, remove=False,
+                       blocking=True):
     LOG.debug('Registering all nodes.')
     ironic_in_use = using_ironic(keystone=None)
     if ironic_in_use:
@@ -216,17 +235,18 @@ def register_all_nodes(service_host, nodes_list, client=None, remove=False):
             LOG.warn('Creating ironic client inline is deprecated, please '
                      'pass the client as parameter.')
             client = clients.get_ironic_client()
-        register_func = _create_or_register_ironic_node
+        register_func = _update_or_register_ironic_node
     else:
         if client is None:
             LOG.warn('Creating nova-bm client inline is deprecated, please '
                      'pass the client as parameter.')
             client = clients.get_nova_bm_client()
-        register_func = _create_or_register_bm_node
+        register_func = _update_or_register_bm_node
     node_map = _populate_node_mapping(ironic_in_use, client)
     seen = set()
     for node in nodes_list:
-        new_node = register_func(service_host, node, node_map, client=client)
+        new_node = register_func(service_host, node, node_map, client=client,
+                                 blocking=blocking)
         seen.add(new_node)
     _clean_up_extra_nodes(ironic_in_use, seen, client, remove=remove)
 
